#include "rf.h"
#include "delay.h"


#define CSN 		P2_0
#define CSN_LOW 	CSN=0
#define CSN_HIGH 	CSN=1

#define MOSI 		P2_1
#define MOSI_LOW 	MOSI=0
#define MOSI_HIGH 	MOSI=1

#define MISO_STATUS P2_2

#define SCK 		P0_7
#define SCK_LOW 	SCK=0
#define SCK_HIGH 	SCK=1


const uint8_t TX_ADDRESS_DEF[4] = {0xCC,0xCC,0xCC,0xCC};
uint8_t temp = 0;

uint8_t TxPayloadLength = PAYLOAD_WIDTH;


/**
  * @˵  	SPIʼ
  *	@	
  * @ֵ 
  * @ע		
  */
void spiInit(void)
{

    P2M0 = P2M0&0x0F|0x80;              //P21
	P0M3 = P0M3&0x0F|0x80;				//P07
	P2M0 = P2M0&0xF0|0x08;				//P20
    P2M1 = P2M1&0xF0|0x04;              //P22SMT
	MISO_STATUS = 1;

   	CSN=1;
	delay_ms(10);
}

/**
  * @˵  	spiRWддһBYTEͬʱһBYTE
  *	@	reg: дBYTE
  * @ֵ BYTE
  * @ע		
  */
uint8_t spiRW(uint8_t reg)
{
	uint8_t	  i;
    for(i = 0; i < 8; i++)
    {
        SCK_LOW;
        if(reg & 0x80)
        {
            MOSI_HIGH;
        }
        else
        {
            MOSI_LOW;
        }
        reg = reg << 1;
        SCK_HIGH;
        if( MISO_STATUS )
        {
          reg = reg | 0x01;
        }
    }
    SCK_LOW;
    return reg;
	
}

void RF_WriteReg( uint8_t reg,  uint8_t wdata)
{
    CSN_LOW;
    spiRW(reg);
    spiRW(wdata);
    CSN_HIGH;
}

uint8_t RF_ReadReg(uint8_t reg)
{
	uint8_t tmp;
	
	CSN_LOW;
	spiRW(reg);
	tmp = spiRW(0);
	CSN_HIGH;

    return tmp;
}

void RF_WriteBuf( uint8_t reg, uint8_t *pBuf, uint8_t length)
{
	uint8_t j;
	CSN_LOW;
	j = 0;
	spiRW(reg);
	for(j = 0;j < length; j++)
	{
	spiRW(pBuf[j]);
	}
	j = 0;
	CSN_HIGH;
}

void RF_ReadBuf( uint8_t reg, unsigned char *pBuf,  uint8_t length)
{
    uint8_t byte_ctr;

    CSN_LOW;                    		                               			
    spiRW(reg);       		                                                		
    for(byte_ctr=0;byte_ctr<length;byte_ctr++)
    	pBuf[byte_ctr] = spiRW(0);                                                 		
    CSN_HIGH;                                                                 		
}


void RF_TxMode(void) //ϵ߻ʱ
{
    
    RF_WriteReg(W_REGISTER + CONFIG,  0X8E);							// RFóTXģʽ
    delay_ms(10);   
    CE_HIGH;	
    delay_ms(10);
}

void RF_RxMode(void) //ϵ߻ʱ
{
    RF_WriteReg(W_REGISTER + CONFIG,  0X8F );							// RFóRXģʽ   			                                       
    delay_ms(10);   
    CE_HIGH;	
    delay_ms(10);
}


void RF_Set_TxMode(void)  //豸ϵɣʼлģʽ
{
    
    RF_WriteReg(W_REGISTER + CONFIG,  0X8E);							// RFóTXģʽ 
    CE_HIGH;	
}

void RF_Set_RxMode(void)  //豸ϵɣʼлģʽ
{
    RF_WriteReg(W_REGISTER + CONFIG,  0X8F );							// RFóRXģʽ   			                                         
    CE_HIGH;	
}

uint8_t ucRF_GetStatus(void)
{
    return RF_ReadReg(STATUS)&0x70;								//ȡRF״̬ 
}

void RF_ClearStatus(void)
{
    RF_WriteReg(W_REGISTER + STATUS,0x70);							//RFIRQ־ 
}

void RF_ClearFIFO(void)
{
    RF_WriteReg(FLUSH_TX, 0);			                                		//RF  TX FIFO		
    RF_WriteReg(FLUSH_RX, 0);                                                   		//RF  RX FIFO	
}

void RF_SetChannel( uint8_t Channel)
{    
 //   CE_LOW;
    RF_WriteReg(W_REGISTER + RF_CH, Channel);
}

void ucRF_TxData( uint8_t *ucPayload,  uint8_t length)
{
    RF_WriteBuf(W_TX_PAYLOAD, ucPayload, length);                               		//write data to txfifo                                                                      		//rf entery tx mode start send data                                                                       		//rf entery stb3
    delay_ms(2);
}

uint8_t ucRF_DumpRxData( uint8_t *ucPayload,  uint8_t length)

{ 
    if(!(ucRF_GetStatus()&&RX_DR_FLAG))
    {
      return 0;                                                                 		
    }
    RF_ReadBuf(R_RX_PAYLOAD, ucPayload, length);                                		//յݶucPayloadrxfifo

    return 1;
}


void RF_TX_CheckResult(uint8_t *ucAckPayload, uint8_t length)
{
  switch(ucRF_GetStatus())
  {
  case TX_DS_FLAG: 
         RF_ClearFIFO();
         RF_ClearStatus();
         break;
           
  case RX_TX_FLAG:
         RF_ReadBuf(R_RX_PAYLOAD,ucAckPayload,length);
         RF_ClearFIFO();
         RF_ClearStatus();
         break;
    
  case MAX_RT_FLAG:
         RF_ClearFIFO();
         RF_ClearStatus();
         break;
  default:
         break;
  }
}

void RF_Init(void)
{
#if(DATA_RATE == DR_1M) 
    //uint8_t BB_cal_data[]    = {0x2a,0xEc,0x6F,0x9c,0x46};   
    uint8_t  BB_cal_data[]    = {0x0A,0x6D,0x67,0x9C,0x46}; 
    uint8_t  RF_cal_data[]    = {0xF6,0x37,0x5D};                                         
    uint8_t  RF_cal2_data[]   = {0xD5,0x21,0xef,0x2C,0x5A,0x40};
    uint8_t  Dem_cal_data[]   = {0x01};  
    uint8_t  Dem_cal2_data[]  = {0x0b,0xDF,0x02};   
#elif(DATA_RATE == DR_250K) 
   uint8_t   BB_cal_data[]    = {0x12,0xec,0x6f,0xa1,0x46}; 
   uint8_t   RF_cal_data[]    = {0xf6,0x37,0x5d};
   uint8_t   RF_cal2_data[]   = {0xd5,0x21,0xeb,0x2c,0x5a,0x40};
   uint8_t   Dem_cal_data[]   = {0x1F};  
   uint8_t   Dem_cal2_data[]  = {0x0b,0xdf,0x02};
#endif
    uint8_t feature = 0x20;
    spiInit();
     delay_ms(10);
	RF_WriteReg(W_REGISTER + FEATURE, feature);					//ʹCE
	                
    RF_WriteReg(RST_FSPI, 0x5A);								//Software Reset    			
    RF_WriteReg(RST_FSPI, 0XA5);
    
    RF_WriteReg(FLUSH_TX, 0);									// CLEAR TXFIFO		    			 
    RF_WriteReg(FLUSH_RX, 0);									// CLEAR  RXFIFO
    RF_WriteReg(W_REGISTER + STATUS, 0x70);							// CLEAR  STATUS	
    RF_WriteReg(W_REGISTER + EN_RXADDR, 0x01);							// Enable Pipe0
    temp = RF_ReadReg(EN_RXADDR);
    RF_WriteReg(W_REGISTER + SETUP_AW,  0x02);							// address witdth is 5 bytes
    RF_WriteReg(W_REGISTER + RF_CH,     DEFAULT_CHANNEL);                                       // 2478M HZ
    RF_WriteReg(W_REGISTER + RX_PW_P0, PAYLOAD_WIDTH );						// 8 bytes
    RF_WriteBuf(W_REGISTER + TX_ADDR,   ( uint8_t*)TX_ADDRESS_DEF, sizeof(TX_ADDRESS_DEF));	// Writes TX_Address to PN006
    RF_WriteBuf(W_REGISTER + RX_ADDR_P0,( uint8_t*)TX_ADDRESS_DEF, sizeof(TX_ADDRESS_DEF));	// RX_Addr0 same as TX_Adr for Auto.Ack  
  
    
    RF_WriteBuf(W_REGISTER + BB_CAL,    BB_cal_data,  sizeof(BB_cal_data));
    RF_WriteBuf(W_REGISTER + RF_CAL2,   RF_cal2_data, sizeof(RF_cal2_data));
    RF_WriteBuf(W_REGISTER + DEM_CAL,   Dem_cal_data, sizeof(Dem_cal_data));
    RF_WriteBuf(W_REGISTER + RF_CAL,    RF_cal_data,  sizeof(RF_cal_data));
    RF_WriteBuf(W_REGISTER + DEM_CAL2,  Dem_cal2_data,sizeof(Dem_cal2_data));
    RF_WriteReg(W_REGISTER + DYNPD, 0x00);					                            
    RF_WriteReg(W_REGISTER + RF_SETUP,  RF_POWER);						// 13DBM  		
    RF_WriteReg(ACTIVATE, 0x73);   
#if(TRANSMIT_TYPE == TRANS_ENHANCE_MODE)      
    RF_WriteReg(W_REGISTER + SETUP_RETR,0x01);							//  1 retrans... 	
    RF_WriteReg(W_REGISTER + EN_AA,     0x01);							// Enable Auto.Ack:Pipe0  	
#elif(TRANSMIT_TYPE == TRANS_BURST_MODE)                                                                
    RF_WriteReg(W_REGISTER + SETUP_RETR,0x00);							// Disable retrans... 	
    RF_WriteReg(W_REGISTER + EN_AA,     0x00);							// Disable AutoAck 
#endif

#if(EN_DYNPLOAD == 1)
    feature |= 0x04;
    RF_WriteReg(W_REGISTER + DYNPD, 0x01); //̬ʹpipe0̬ 
#endif

#if(EN_ACK_PAYLOAD == 1)
    feature |= 0x02;  
#endif

if(PAYLOAD_WIDTH >32)
    feature |= 0x18;

RF_WriteReg(W_REGISTER + FEATURE, feature);//ʹACKPAYLOAD



}

/******************************************************************************/
//            		زģʽ
/******************************************************************************/
void RF_Carrier( uint8_t ucChannel_Set)
{
    uint8_t BB_cal_data[]    = {0x0A,0x6D,0x67,0x9C,0x46}; 
    uint8_t RF_cal_data[]    = {0xF6,0x37,0x5D};
    uint8_t RF_cal2_data[]   = {0x45,0x21,0xEF,0xAC,0x5a,0x50};
    uint8_t Dem_cal_data[]   = {0xE1}; 								
    uint8_t Dem_cal2_data[]  = {0x0B,0xDF,0x02};      

	RF_WriteReg(W_REGISTER + FEATURE, 0x20);			//ʹCE

    CE_LOW;
    delay_ms(5);				 						//delay 500us
    RF_WriteReg(W_REGISTER + CONFIG, 0X8e);        						//tx mode 
    RF_WriteReg(W_REGISTER + RF_CH, ucChannel_Set);						//زƵ	   
    RF_WriteReg(W_REGISTER + RF_SETUP, RF_POWER);      						//13dbm
    RF_WriteBuf(W_REGISTER + BB_CAL,    BB_cal_data,  sizeof(BB_cal_data));
    RF_WriteBuf(W_REGISTER + RF_CAL2,   RF_cal2_data, sizeof(RF_cal2_data));
    RF_WriteBuf(W_REGISTER + DEM_CAL,   Dem_cal_data, sizeof(Dem_cal_data));
    RF_WriteBuf(W_REGISTER + RF_CAL,    RF_cal_data,  sizeof(RF_cal_data));
    RF_WriteBuf(W_REGISTER + DEM_CAL2,  Dem_cal2_data,sizeof(Dem_cal2_data));
    delay_ms(5);	
}

void RF_Sleep()
{
 unsigned char reg_config;
 reg_config = RF_ReadReg(CONFIG);
 RF_WriteReg(W_REGISTER + CONFIG, reg_config & (~(0x02))); 
 CE_LOW;
}

void RF_awaken()
{
 unsigned char reg_config;
 reg_config = RF_ReadReg(CONFIG);
 RF_WriteReg(W_REGISTER + CONFIG, reg_config | 0x02); 
 CE_HIGH;
 delay_ms(7);
}

uint8_t RF_RxText(void)
{

}